#include "tcpip.h"

struct realm *udp_realm;

struct tcp_head *udp_rules=0;
unsigned int num_udp_head=0;
unsigned int num_udp_rules=0;

void udp_match(struct packet *p, unsigned int l)
{
	struct tcp_head *h;
	struct pkt_iphdr *iph;
	struct pkt_udphdr *udph;
	u_int16_t sp, dp;

	iph=p->layer[l].h.ip;
	udph=p->layer[l+1].h.udp;

	sp=ntohs(udph->sport);
	dp=ntohs(udph->dport);

	for(h=udp_rules; h; h=h->next) {
		if ( 
			(((h->saddr&h->smask) == (iph->saddr&h->smask))
			^ ((h->flags&NEGATE_SADDR)!=0) ) &&
			
			(((h->daddr&h->dmask) == (iph->daddr&h->dmask))
			^ ((h->flags&NEGATE_DADDR)!=0) ) &&

			(((sp>=h->sport_min)&&(sp<=h->sport_max)))
			^ ((h->flags&NEGATE_SPORT)!=0) &&
			
			(((dp>=h->dport_min)&&(dp<=h->dport_max)))
			^ ((h->flags&NEGATE_DPORT)!=0)){

			tcpip_traverse(p, l, h->rules);
		}
	}
}

int udp_commit(struct rule *r)
{
	return tcp_udp_commit(&udp_rules,
		&num_udp_head,
		&num_udp_rules, r);
}


struct proto udp_p=init_proto_ex("udp", udp_decode, udp_dprint);
struct proto_req udp_r[]={
	proto_request("ipv4", 17),
	null_request()
};

int udp_dprint(struct layer *l, char *buf, int buflen)
{
	return snprintf(buf, buflen,
		"SPT=%u DPT=%u LEN=%u CSUM=0x%x",
		htons(l->h.udp->sport),
		htons(l->h.udp->dport),
		htons(l->h.udp->len),
		htons(l->h.udp->csum));
}

int udp_decode(struct packet *p)
{
	struct proto_child *pc;
	struct layer *l=&p->layer[p->llen-1];
	
	if ( p->llen >= PKT_LAYERS ) return 0;

	if ( (p->layer[p->llen].h.raw=
		l->h.raw+sizeof(pkt_udphdr_t)) 
		>= p->end ) goto finish;

	for(pc=l->proto->children; pc; pc=pc->next)
	{
		if ( l->h.udp->dport == pc->id ||
			l->h.udp->sport == pc->id ) {
			p->layer[p->llen++].proto=pc->proto;
			return pc->proto->decode(p);
		}
	}

finish:
	p->layer[p->llen++].proto=NULL;
	return 0;
}
